@@ -27,25 +27,45 @@
 
 class Matrix;
 
-class Bezier {
+class QuadBezier {
 	public:
-		Bezier ();
-		Bezier (const DPair &p0, const DPair &p1, const DPair &p2);
-		Bezier (const DPair &p0, const DPair &p1, const DPair &p2, const DPair &p3);
-		Bezier (const Bezier &source, double t0, double t1);
+		QuadBezier ();
+		QuadBezier (const DPair &p0, const DPair &p1, const DPair &p2);
+		void setPoints (const DPair &p0, const DPair &p1, const DPair &p2);
+		constexpr const DPair& point (int i) const {return _points[i];}
+		DPair valueAt (double t) const;
+		DPair derivativeAt (double t) const;
+		double arclen (double t=1.0) const;
+
+	private:
+		DPair _points[3];
+};
+
+
+class CubicBezier {
+	public:
+		CubicBezier ();
+		//CubicBezier (const DPair &p0, const DPair &p1, const DPair &p2);
+		CubicBezier (const DPair &p0, const DPair &p1, const DPair &p2, const DPair &p3);
+		CubicBezier (double x0, double y0, double x1, double y1, double x2, double y2, double x3, double y3);
+		CubicBezier (const CubicBezier &source, double t0, double t1);
+		explicit CubicBezier (const QuadBezier &qbezier);
 		void setPoints (const DPair &p0, const DPair &p1, const DPair &p2, const DPair &p3);
 		void reverse ();
 		DPair valueAt (double t) const;
 		DPair blossomValue (double u, double v, double w) const;
-		void subdivide (double t, Bezier *bezier1, Bezier *bezier2) const;
-		Bezier& transform (const Matrix &matrix);
+		void subdivide (double t, CubicBezier *bezier1, CubicBezier *bezier2) const;
+		CubicBezier& transform (const Matrix &matrix);
 		int approximate (double delta, std::vector<DPair> &p, std::vector<double> *t=nullptr) const;
-		const DPair& point (int i) const {return _points[i];}
+		constexpr const DPair& point (int i) const {return _points[i];}
 		int reduceDegree (double delta, std::vector<DPair> &p) const;
+		std::vector<QuadBezier> toQuadBeziers (double delta, std::vector<double> *startParams=nullptr) const;
 		BoundingBox getBBox () const;
 
 	protected:
 		int approximate (double delta, double t0, double t1, std::vector<DPair> &p, std::vector<double> *t) const;
+		void toQuadBeziers (double t0, double t1, double precision, std::vector<QuadBezier> &qbeziers, std::vector<double> *startParams) const;
+		QuadBezier midpointApproximation () const;
 
 	private:
 		DPair _points[4];
