@@ -2,7 +2,7 @@
 ** DVIToSVG.cpp                                                         **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -350,9 +350,7 @@ void DVIToSVG::embedFonts (XMLElement *svgElement) {
 	if (!svgElement || !_actions) // no dvi actions => no chars written => no fonts to embed
 		return;
 
-	const DVIToSVGActions *svgActions = static_cast<DVIToSVGActions*>(_actions.get());
-	auto &usedCharsMap = svgActions->getUsedChars();
-
+	auto &usedCharsMap = FontManager::instance().getUsedChars();
 	collect_chars(usedCharsMap);
 
 	GlyphTracerMessages messages;
@@ -374,7 +372,7 @@ void DVIToSVG::embedFonts (XMLElement *svgElement) {
 		else
 			Message::wstream(true) << "can't embed font '" << font->name() << "'\n";
 	}
-	_svg.appendFontStyles(svgActions->getUsedFonts());
+	_svg.appendFontStyles(FontManager::instance().getUsedFonts());
 }
 
 
@@ -569,16 +567,18 @@ void DVIToSVG::HashSettings::setParameters (const string &paramstr) {
 		auto it = paramMap.find(name);
 		if (it != paramMap.end())
 			_params.insert(it->second);
-		else if (_algo.empty() && HashFunction::isSupportedAlgorithm(name))
-			_algo = name;
-		else if (!name.empty()) {
-			string msg = "invalid hash parameter '"+name+"' (supported algorithms: ";
-			for (string str : HashFunction::supportedAlgorithms())
-				msg += str + ", ";
-			msg.pop_back();
-			msg.pop_back();
-			msg += ')';
-			throw MessageException(msg);
+		else if (_algo.empty()) {
+			if (HashFunction::isSupportedAlgorithm(name))
+				_algo = name;
+			else if (!name.empty()) {
+				string msg = "invalid hash parameter '" + name + "' (supported algorithms: ";
+				for (string str: HashFunction::supportedAlgorithms())
+					msg += str + ", ";
+				msg.pop_back();
+				msg.pop_back();
+				msg += ')';
+				throw MessageException(msg);
+			}
 		}
 	}
 	// set default hash algorithm if none is given
