@@ -28,7 +28,7 @@ class FileSystem {
 	class TemporaryDirectory {
 		friend class FileSystem;
 		public:
-			TemporaryDirectory (const std::string &folder, std::string prefix);
+			TemporaryDirectory (const std::string &folder, std::string prefix, bool inplace=false);
 			TemporaryDirectory (TemporaryDirectory &&tmpdir) =default;
 			~TemporaryDirectory ();
 			TemporaryDirectory& operator = (TemporaryDirectory &&tmpdir) =default;
@@ -47,11 +47,12 @@ class FileSystem {
 		static bool copy (const std::string &src, const std::string &dest, bool remove_src=false);
 		static uint64_t filesize (const std::string &fname);
 		static std::string ensureForwardSlashes (std::string path);
+		static std::string ensureSystemSlashes (std::string path);
 		static std::string getcwd ();
 #ifdef _WIN32
 		static std::string getcwd (char drive);
 #endif
-		static std::string tmpdir ();
+		static std::string tmpdir (bool inplace=false);
 		static bool chdir (const std::string &dir);
 		static bool exists (const std::string &fname);
 		static bool mkdir (const std::string &dirname);
