@@ -2,7 +2,7 @@
 ** Ghostscript.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2021 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2022 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -228,10 +228,13 @@ bool Ghostscript::revision (gsapi_revision_t *r) const {
 
 /** Returns the revision number of the GS library. */
 int Ghostscript::revision () const {
-	gsapi_revision_t r;
-	if (revision(&r))
-		return static_cast<int>(r.revision);
-	return 0;
+	static int rev=0;
+	if (rev == 0) {
+		gsapi_revision_t r;
+		if (revision(&r))
+			rev = static_cast<int>(r.revision);
+	}
+	return rev;
 }
 
 
