@@ -66,8 +66,7 @@ string MiKTeXCom::getVersion () {
 	MiKTeXSetupInfo info;
 	_session->GetMiKTeXSetupInfo(&info);
 #endif
-	_bstr_t version = info.version;
-	return string(version);
+	return to_string(info.series/100)+"."+ to_string(info.series%100);
 }
 
 
@@ -105,6 +104,6 @@ const char* MiKTeXCom::findFile (const char *fname) {
 		return nullptr;
 	}
 	catch (_com_error &e) {
-		throw MessageException((const char*)e.Description());
+		throw MessageException(static_cast<const char*>(e.Description()));
 	}
 }
