@@ -23,40 +23,27 @@
 
 #include <fstream>
 #include "ImageToSVG.hpp"
+#include "PDFHandler.hpp"
 
-class PsSpecialHandler;
 
 class PDFToSVG : public ImageToSVG {
 	public:
-		PDFToSVG (const std::string &fname, SVGOutputBase &out) : ImageToSVG(fname, out) {}
+		PDFToSVG (const std::string &fname, SVGOutputBase &out);
 		bool isSinglePageFormat() const override {return false;}
-
-		/** Returns the total number of pages in the PDF file. */
-		int totalPageCount() const override {
-			if (_totalPageCount < 0) {
-				_totalPageCount = psInterpreter().pdfPageCount(filename());
-				if (_totalPageCount < 1)
-					throw MessageException("can't retrieve number of pages from file " + filename());
-			}
-			return _totalPageCount;
-		}
+		int totalPageCount() const override;
+		void convert (int pageno) override;
 
 	protected:
-		bool imageIsValid () const override {
-			std::ifstream ifs(filename());
-			if (ifs) {
-				char buf[16];
-				ifs.getline(buf, 16);
-				return std::strncmp(buf, "%PDF-1.", 7) == 0;
-			}
-			return false;
-		}
+		void checkGSAndFileFormat () override;
+		bool imageIsValid () const override;
 		std::string imageFormat () const override {return "PDF";}
-		BoundingBox imageBBox () const override {return BoundingBox();}
+		BoundingBox imageBBox () const override {return {};}
 		std::string psSpecialCmd () const override {return "pdffile=";}
 
 	private:
 		mutable int _totalPageCount = -1;
+		PDFHandler _pdfHandler;
+		bool _useGS = true;
 };
 
 #endif
