@@ -23,13 +23,30 @@
 
 #include <string>
 
+struct SearchPattern {
+	SearchPattern () =default;
+
+	explicit SearchPattern (std::string searchRegex)
+		: search(std::move(searchRegex)) {}
+
+	SearchPattern (std::string searchRegex, std::string replExpr)
+		: search(std::move(searchRegex)), replace(std::move(replExpr)) {}
+
+	std::string search;
+	std::string replace;
+};
+
+
 class Process {
 	public:
+		enum PipeFlags {PF_STDOUT=1, PF_STDERR=2};
+
 		Process (std::string cmd, std::string paramstr);
 		Process (const Process &orig) =delete;
 		Process (Process &&orig) =delete;
-		bool run (std::string *out=nullptr);
-		bool run (const std::string &dir, std::string *out=nullptr);
+		bool run (std::string *out=nullptr, PipeFlags flags=PF_STDOUT);
+		bool run (const std::string &dir, std::string *out=nullptr, PipeFlags flags=PF_STDOUT);
+		bool run (std::string *out, const SearchPattern &pattern, PipeFlags flags=PF_STDOUT);
 
 	private:
 		std::string _cmd;
