@@ -27,6 +27,7 @@
 #include <string>
 #include <vector>
 #include "GraphicsPath.hpp"
+#include "PDFHandler.hpp"
 #include "PSInterpreter.hpp"
 #include "Opacity.hpp"
 #include "PSPattern.hpp"
@@ -76,6 +77,14 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 			std::stack<Entry> _stack;  ///< stack holding the clipping information of the current graphics context
 	};
 
+	struct ImageNode {
+		ImageNode () : matrix(1) {}
+		explicit ImageNode (std::unique_ptr<SVGElement> elem) : element(std::move(elem)), matrix(1) {}
+		ImageNode (std::unique_ptr<SVGElement> elem, const Matrix &m) : element(std::move(elem)), matrix(m) {}
+		std::unique_ptr<SVGElement> element;
+		Matrix matrix;
+	};
+
 	enum PsSection {PS_NONE, PS_HEADERS, PS_BODY};
 	enum class FileType {EPS, PDF, SVG, BITMAP};
 
@@ -97,6 +106,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		static int SHADING_SEGMENT_SIZE;
 		static double SHADING_SIMPLIFY_DELTA;
 		static std::string BITMAP_FORMAT;
+		static bool EMBED_BITMAP_DATA;
 
 	protected:
 		void initialize ();
@@ -105,7 +115,10 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		void executeAndSync (std::istream &is, bool updatePos);
 		void processHeaderFile (const char *fname);
 		void imgfile (FileType type, const std::string &fname, const std::map<std::string,std::string> &attr);
-		std::unique_ptr<SVGElement> createImageNode (FileType type, const std::string &fname, int pageno, BoundingBox bbox, bool clip);
+		ImageNode createImageNode (FileType type, const std::string &fname, int pageno, BoundingBox bbox, bool clip);
+		ImageNode createBitmapNode (const std::string &fname, const std::string &path, int pageno, BoundingBox bbox);
+		ImageNode createPSNode (const std::string &fname, const std::string &path, int pageno, BoundingBox bbox, bool clip);
+		ImageNode createPDFNode (const std::string &fname, const std::string &path, int pageno, BoundingBox bbox, bool clip);
 		void dviBeginPage (unsigned int pageno, SpecialActions &actions) override;
 		void dviEndPage (unsigned pageno, SpecialActions &actions) override;
 		void clip (Path path, bool evenodd);
@@ -164,6 +177,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 
 	private:
 		PSInterpreter _psi;
+		PDFHandler _pdfHandler;
 		SpecialActions *_actions=nullptr;
 		PSPreviewFilter _previewFilter;    ///< filter to extract information generated by the preview package
 		PsSection _psSection=PS_NONE;      ///< current section processed (nothing yet, headers, or body specials)
@@ -189,6 +203,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		std::map<int, std::unique_ptr<PSPattern>> _patterns;
 		PSTilingPattern *_pattern;         ///< current pattern
 		bool _patternEnabled;              ///< true if active color space is a pattern
+		std::string _pdfProc;              ///< tool to process PDF files ("gs" or "mutool")
 };
 
 #endif
